@@ -1,9 +1,12 @@
 from fastapi import Depends, APIRouter
 from fastapi.responses import ORJSONResponse
 from starlette.status import HTTP_409_CONFLICT
+from tortoise import Model
 from tortoise.contrib.pydantic import pydantic_model_creator
 from tortoise.exceptions import IntegrityError
+from tortoise.fields import ManyToManyRelation
 
+from ..common import handle_m2m_fields_create_or_update
 from ..depends import QueryItem, get_query, parse_body, get_model
 from ..factory import app
 from ..responses import GetManyOut
@@ -20,12 +23,20 @@ async def get_resource(
         query: QueryItem = Depends(get_query),
         model=Depends(get_model)
 ):
+    menu = app.model_menu_mapping[resource]
     qs = model.all()
     if query.where:
         qs = qs.filter(**query.where)
-    result = await qs.limit(query.size).offset((query.page - 1) * query.size)
+    sort = query.sort
+    for k, v in sort.items():
+        if k in menu.sort_fields:
+            if v == -1:
+                qs = qs.order_by(f'-{k}')
+            elif v == 1:
+                qs = qs.order_by(k)
     resource = await app.get_resource(resource)
-    creator = pydantic_model_creator(model, include=resource.resource_fields.keys())
+    result = await qs.limit(query.size).offset((query.page - 1) * query.size)
+    creator = pydantic_model_creator(model, include=resource.resource_fields.keys(), exclude=model._meta.m2m_fields)
     return GetManyOut(
         total=await qs.count(),
         data=list(map(lambda x: creator.from_orm(x).dict(), result))
@@ -38,7 +49,7 @@ async def get_resource(
 async def form(
         resource: str,
 ):
-    resource = await app.get_resource(resource, exclude_readonly=True)
+    resource = await app.get_resource(resource, exclude_pk=True, exclude_m2m_field=False)
     return resource.dict(by_alias=True, exclude_unset=True)
 
 
@@ -82,16 +93,15 @@ async def update_one(
         model=Depends(get_model)
 ):
     body, resource_fields = parsed
+    m2m_fields = model._meta.m2m_fields
     try:
-        print(body)
-        await model.filter(pk=id).update(**body)
+        obj = await handle_m2m_fields_create_or_update(body, m2m_fields, model, False, id)
     except IntegrityError as e:
         return ORJSONResponse(status_code=HTTP_409_CONFLICT, content=dict(
             message=f'Update Error,{e}'
         ))
-    user_ = await get_object_or_404(model, pk=id)
-    creator = pydantic_model_creator(model, include=resource_fields)
-    return creator.from_orm(user_).dict()
+    creator = pydantic_model_creator(model, include=resource_fields, exclude=m2m_fields)
+    return creator.from_orm(obj).dict()
 
 
 @router.post(
@@ -102,9 +112,10 @@ async def create_one(
         model=Depends(get_model)
 ):
     body, resource_fields = parsed
-    creator = pydantic_model_creator(model, include=resource_fields)
+    m2m_fields = model._meta.m2m_fields
+    creator = pydantic_model_creator(model, include=resource_fields, exclude=m2m_fields)
     try:
-        obj = await model.create(**body)
+        obj = await handle_m2m_fields_create_or_update(body, m2m_fields, model)
     except IntegrityError as e:
         return ORJSONResponse(status_code=HTTP_409_CONFLICT, content=dict(
             message=f'Create Error,{e}'
@@ -120,8 +131,15 @@ async def get_one(
         resource: str,
         model=Depends(get_model)
 ):
-    obj = await get_object_or_404(model, pk=id)
-    resource = await app.get_resource(resource)
+    obj = await get_object_or_404(model, pk=id)  # type:Model
+    m2m_fields = model._meta.m2m_fields
+    resource = await app.get_resource(resource, exclude_m2m_field=False)
     include = resource.resource_fields.keys()
-    creator = pydantic_model_creator(model, include=include)
-    return creator.from_orm(obj).dict()
+    creator = pydantic_model_creator(model, include=include, exclude=m2m_fields)
+    ret = creator.from_orm(obj).dict()
+    for m2m_field in m2m_fields:
+        if m2m_field in include:
+            relate_model = getattr(obj, m2m_field)  # type:ManyToManyRelation
+            ids = await relate_model.all().values_list(relate_model.model._meta.pk_attr)
+            ret[m2m_field] = list(map(lambda x: x[0], ids))
+    return ret
